using System;
using SlatedGameToolkit.Tools.System.Interaction;

namespace SlatedGameToolkit.Tools.System
{
    public class CommandProcessor
    {
        private string generalHelpMessage;
        CommandMap commandMap;

        /// <summary>
        /// The general help is the string that is printed when the input is not understood.
        /// {input} in the string is replaced with the user input.
        /// </summary>
        /// <param name="generalHelp"></param>
        /// <param name="commands"></param>
        public CommandProcessor(string generalHelp, CommandMap commands) {
            this.commandMap = commands;
            this.generalHelpMessage = generalHelp;
        }
        public void Process(IInteractable interactable) {
            string message = interactable.Listen();
            string[] splitMessage = message.Split(' ');
            string invocation = splitMessage[0];

            IInvocable invocable = commandMap[invocation];
            if (invocable != null) {
                string[] args = new string[splitMessage.Length - 1];
                Array.Copy(splitMessage, 1, args, 0, splitMessage.Length - 1);
                if (invocable.Execute(interactable, args)) {
                    return;
                }
            }
            interactable.Tell(generalHelpMessage.Replace("{input}", invocation));
        }
    }
}